# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import pathlib
import sys


print("python exec:", sys.executable)
print("sys.path:", sys.path)
root = pathlib.Path(__file__).parent.parent.absolute()
os.environ["PYTHONPATH"] = str(root)
sys.path.insert(0, str(root))

import ocetrac  # isort:skip

# -- Project information -----------------------------------------------------

project = "ocetrac"
copyright = "2021, ocetrac"
author = "ocetrac"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
# see https://pypi.org/project/setuptools-scm/ for details
from pkg_resources import get_distribution
release = get_distribution('ocetrac').version
# for example take major/minor
version = '.'.join(release.split('.')[:2])

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "nbsphinx",
    "recommonmark",
    "sphinx.ext.mathjax",
    "sphinx.ext.autosummary",
    "sphinx.ext.extlinks",
    "sphinx.ext.intersphinx",
    "numpydoc",
    "nbsphinx",
    "IPython.sphinxext.ipython_directive",
    "IPython.sphinxext.ipython_console_highlighting",
    "sphinxcontrib.srclinks",
]

autosummary_generate = True

autodoc_default_flags = ["members", "inherited-members"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix of source filenames.
source_suffix = [".rst", ".md"]

napoleon_numpy_docstring = True

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = u"ocetrac"
copyright = u"ocetrac, 2021"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = ".".join(version.split(".")[:2])
# The full version, including alpha/beta/rc tags.
release = version

# link to github issues
extlinks = {
    "issue": ("https://github.com/ocetrac/ocetrac/issues/%s", "GH#"),
    "pull": ("https://github.com/ocetrac/ocetrac/issues/%s", "GH#"),
}

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "_images"]#, "**.ipynb_checkpoints", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# To get SVG and PDF plots for nbsphinx
nbsphinx_execute_arguments = [
    "--InlineBackend.figure_formats={'svg', 'pdf'}",
    "--InlineBackend.rc=figure.dpi=96",
]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme" #"pangeo"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    "logo_only": True,
    "display_version": False,
    "style_nav_header_background": "#343131",
}

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = "img/tranparent_logo.png"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Output file base name for HTML help builder.
htmlhelp_basename = "ReadtheDocsTemplatedoc"

# -- nbsphinx specific options ----------------------------------------------
# this allows notebooks to be run even if they produce errors.
nbsphinx_allow_errors = True

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        "index",
        "ReadtheDocsTemplate.tex",
        u"Read the Docs Template Documentation",
        u"Read the Docs",
        "manual",
    ),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        "index",
        "readthedocstemplate",
        u"Read the Docs Template Documentation",
        [u"Read the Docs"],
        1,
    )
]

# If true, show URL addresses after external links.
# m an_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        "index",
        "ReadtheDocsTemplate",
        u"Read the Docs Template Documentation",
        u"Read the Docs",
        "ReadtheDocsTemplate",
        "Miscellaneous",
    ),
]


# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
# texinfo_no_detailmenu = False







"""
This module applies all Ocetrac methods for labeling objects in 3 dimensions.
"""
import dask.array as dsa
import numpy as np
import scipy.ndimage
import xarray as xr

from skimage.measure import label as label_np
from skimage.measure import regionprops


def _apply_mask(binary_images, mask):
    binary_images_with_mask = binary_images.where(mask == 1, drop=False, other=0)
    return binary_images_with_mask


class Tracker:
    """
    Tracker object for applying Ocetrac
    """

    def __init__(
        self, da, mask, radius, min_size_quartile, timedim, xdim, ydim, positive=True
    ):
        self.da = da
        self.mask = mask
        self.radius = radius
        self.min_size_quartile = min_size_quartile
        self.timedim = timedim
        self.xdim = xdim
        self.ydim = ydim
        self.positive = positive

        if (timedim, ydim, xdim) != da.dims:
            try:
                da = da.transpose(timedim, ydim, xdim)
            except:
                raise ValueError(
                    f"Ocetrac currently only supports 3D DataArrays. The dimensions should only contain ({timedim}, {xdim}, and {ydim}). Found {list(da.dims)}"
                )

    def track(self):
        """
        Label and track image features.

        Parameters
        ----------
        da : xarray.DataArray
            The data to label.

        mask : xarray.DataArray
            The mask of points to ignore. Must be binary where 1 = true point and 0 = background to be ignored.

        radius : int
            The size of the structuring element used in morphological opening and closing. Radius specified by the number of grid units.
            
            Structuring elements are defined such that cells are included if their distance from the origin in index space is
            strictly less than the radius. For example, a radius of 1 means that the structuring element includes just an
            individual pixel (such that applying the morhological closing and opening just returns the original binary array),
            while a radius of 2 would additionally include the eight cells cells adjacent to the origin. As the radius increases,
            the shape of the structure element asymptotes to a circle centered on the origin.

        min_size_quartile : float
            The quantile used to define the threshold of the smallest area object retained in tracking. Value should be between 0 and 1.
            A value of exactly 0 means objects of any size are retained while a value of 1 retains just the single largest event.
            Higher values of `min_size_quartile` result in improved performance because less events are stored in memory and need to be
            compared.

        timedim : str
            The name of the time dimension

        xdim : str
            The name of the x dimension

        ydim : str
            The name of the y dimension

        positive : bool
            True if da values are expected to be positive, false if they are negative. Default argument is True

        Returns
        -------
        labels : xarray.DataArray
            Integer labels of the connected regions.
        """

        if (self.mask == 0).all():
            raise ValueError(
                "Found only zeros in `mask` input. The mask should indicate valid regions with values of 1"
            )

        # Convert data to binary, define structuring element, and perform morphological closing then opening
        binary_images = self._morphological_operations()

        # Apply mask
        binary_images_with_mask = _apply_mask(
            binary_images, self.mask
        )  # perhaps change to method? JB

        # Filter area
        area, min_area, binary_labels, N_initial = self._filter_area(
            binary_images_with_mask
        )

        # Label objects
        labels, num = self._label_either(binary_labels, return_num=True, connectivity=3)

        # Wrap labels
        grid_res = abs(self.da[self.xdim][1] - self.da[self.xdim][0])
        if self.da[self.xdim][-1] - self.da[self.xdim][0] >= 360 - grid_res:
            labels_wrapped, N_final = self._wrap(labels)
        else:
            labels_wrapped = labels
            N_final = np.max(labels)

        # Final labels to DataArray
        new_labels = xr.DataArray(
            labels_wrapped, dims=self.da.dims, coords=self.da.coords
        )
        new_labels = new_labels.where(new_labels != 0, drop=False, other=np.nan)

        ## Metadata

        # Calculate Percent of total object area retained after size filtering
        sum_tot_area = int(np.sum(area.values))

        reject_area = area.where(area <= min_area, drop=True)
        sum_reject_area = int(np.sum(reject_area.values))
        percent_area_reject = sum_reject_area / sum_tot_area

        accept_area = area.where(area > min_area, drop=True)
        sum_accept_area = int(np.sum(accept_area.values))
        percent_area_accept = sum_accept_area / sum_tot_area

        new_labels = new_labels.rename("labels")
        new_labels.attrs["initial objects identified"] = int(N_initial)
        new_labels.attrs["final objects tracked"] = int(N_final)
        new_labels.attrs["radius"] = self.radius
        new_labels.attrs["size quantile threshold"] = self.min_size_quartile
        new_labels.attrs["min area"] = min_area
        new_labels.attrs["percent area reject"] = percent_area_reject
        new_labels.attrs["percent area accept"] = percent_area_accept

        print("initial objects identified \t", int(N_initial))
        print("final objects tracked \t", int(N_final))

        return new_labels

    ### PRIVATE METHODS - not meant to be called by user ###

    def _morphological_operations(self):
        """Converts xarray.DataArray to binary, defines structuring element, and performs morphological closing then opening.
        Parameters
        ----------
        da     : xarray.DataArray
                The data to label
        radius : int
                Length of grid spacing that defines the radius of the structuring element used in morphological closing and opening.
                
                Structuring elements are defined such that cells are included if their distance from the origin in index space is
                strictly less than the radius. For example, a radius of 1 means that the structuring element includes just an
                individual pixel (such that applying the morhological closing and opening just returns the original binary array),
                while a radius of 2 would additionally include the eight cells cells adjacent to the origin. As the radius increases,
                the shape of the structure element asymptotes to a circle centered on the origin.

        """

        # Convert images to binary. All positive values == 1, otherwise == 0
        if self.positive == True:
            bitmap_binary = self.da.where(self.da > 0, drop=False, other=0)

        elif self.positive == False:
            bitmap_binary = self.da.where(self.da < 0, drop=False, other=0)

        bitmap_binary = bitmap_binary.where(bitmap_binary == 0, drop=False, other=1)

        # Define structuring element
        diameter = self.radius * 2
        x = np.arange(-self.radius, self.radius + 1)
        x, y = np.meshgrid(x, x)
        r = x**2 + y**2
        se = r < self.radius**2

        def binary_open_close(bitmap_binary):
            bitmap_binary_padded = np.pad(
                bitmap_binary, ((diameter, diameter), (diameter, diameter)), mode="wrap"
            )
            # If the radius is equal to 1, the structuring element is just an individual pixel, so
            # it is faster to just skip the calls to `binary_closing` and `binary_opening`
            if self.radius == 1:
                s2 = bitmap_binary_padded
            elif self.radius > 1:
                s1 = scipy.ndimage.binary_closing(bitmap_binary_padded, se, iterations=1)
                s2 = scipy.ndimage.binary_opening(s1, se, iterations=1)
            else:
                raise ValueError("radius must be an integer greater than or equal to 1")
                
            unpadded = s2[diameter:-diameter, diameter:-diameter]
            return unpadded

        mo_binary = xr.apply_ufunc(
            binary_open_close,
            bitmap_binary,
            input_core_dims=[[self.ydim, self.xdim]],
            output_core_dims=[[self.ydim, self.xdim]],
            output_dtypes=[bitmap_binary.dtype],
            vectorize=True,
            dask="parallelized",
        )
        return mo_binary

    def _filter_area(self, binary_images):
        """calculatre area with regionprops"""

        def get_labels(binary_images):
            blobs_labels = self._label_either(binary_images, background=0)
            return blobs_labels

        labels = xr.apply_ufunc(
            get_labels,
            binary_images,
            input_core_dims=[[self.ydim, self.xdim]],
            output_core_dims=[[self.ydim, self.xdim]],
            output_dtypes=[binary_images.dtype],
            vectorize=True,
            dask="parallelized",
        )

        labels = xr.DataArray(
            labels, dims=binary_images.dims, coords=binary_images.coords
        )
        labels = labels.where(labels > 0, drop=False, other=np.nan)

        # The labels are repeated each time step, therefore we relabel them to be consecutive
        max_id = 0
        for i in range(1, labels.shape[0]):
            max_id = np.nanmax([max_id, labels[i - 1, :, :].max().values])
            labels[i, :, :] = labels[i, :, :].values + max_id

        labels = labels.where(labels > 0, drop=False, other=0)
        labels_wrapped, N_initial = self._wrap(np.array(labels))

        # Calculate Area of each object and keep objects larger than threshold
        props = regionprops(labels_wrapped.astype("int"))

        labelprops = [p.label for p in props]
        labelprops = xr.DataArray(
            labelprops, dims=["label"], coords={"label": labelprops}
        )

        area = xr.DataArray(
            [p.area for p in props], dims=["label"], coords={"label": labelprops}
        )  # Number of pixels of the region.

        if area.size == 0:
            raise ValueError(
                f"No objects were detected. Try changing radius or min_size_quartile parameters."
            )

        min_area = np.percentile(area, self.min_size_quartile * 100)
        print(f"minimum area: {min_area}")

        keep_labels = labelprops.where(area >= min_area, drop=True)
        keep_where = np.isin(labels_wrapped, keep_labels)
        out_labels = xr.DataArray(
            np.where(keep_where == False, 0, labels_wrapped),
            dims=binary_images.dims,
            coords=binary_images.coords,
        )

        # Convert images to binary. All positive values == 1, otherwise == 0
        binary_labels = out_labels.where(out_labels == 0, drop=False, other=1)

        return area, min_area, binary_labels, N_initial

    def _label_either(self, data, **kwargs):
        if isinstance(data, dsa.Array):
            try:
                from dask_image.ndmeasure import label as label_dask

                def label_func(a, **kwargs):
                    ids, num = label_dask(a, **kwargs)
                    return ids

            except ImportError:
                raise ImportError(
                    "Dask_image is required to use this function on Dask arrays. "
                    "Either install dask_image or else call .load() on your data."
                )
        else:
            label_func = label_np
        return label_func(data, **kwargs)

    def _wrap(self, labels):
        """Impose periodic boundary and wrap labels"""
        first_column = labels[..., 0]
        last_column = labels[..., -1]

        unique_first = np.unique(first_column[first_column > 0])

        # This loop iterates over the unique values in the first column, finds the location of those values in
        # the first columnm and then uses that index to replace the values in the last column with the first column value
        for i in enumerate(unique_first):
            first = np.where(first_column == i[1])
            last = last_column[first[0], first[1]]
            bad_labels = np.unique(last[last > 0])
            replace = np.isin(labels, bad_labels)
            labels[replace] = i[1]

        labels_wrapped = np.unique(labels, return_inverse=True)[1].reshape(labels.shape)

        # recalculate the total number of labels
        N = np.max(labels_wrapped)

        return labels_wrapped, N



import dask.array as dsa
import numpy as np
import pytest
import scipy.ndimage
import xarray as xr

from skimage.measure import label as label_np
from skimage.measure import regionprops

from ocetrac import Tracker, _apply_mask


@pytest.fixture
def example_data():
    x0 = [180, 225, 360, 80, 1, 360, 1]
    y0 = [0, 20, -50, 40, -50, 40, 40]
    sigma0 = [15, 25, 30, 10, 30, 15, 10]

    lon = np.arange(0, 360) + 0.5
    lat = np.arange(-90, 90) + 0.5
    x, y = np.meshgrid(lon, lat)
    timedim = "time"
    xdim = "lon"
    ydim = "lat"

    def make_blobs(x0, y0, sigma0):
        blob = np.exp(-((x - x0) ** 2 + (y - y0) ** 2) / (2 * sigma0**2))
        return blob

    features = {}
    for i in range(len(x0)):
        features[i] = make_blobs(x0[i], y0[i], sigma0[i])

    first_image = features[0] + features[1] + features[3] - 0.5

    da = xr.DataArray(
        first_image[np.newaxis, :, :],
        dims=[timedim, ydim, xdim],
        coords={timedim: [1], ydim: lat, xdim: lon},
    )

    da_shift_01 = da.shift(lon=0, lat=-20, fill_value=-0.5)
    da_shift_02 = da.shift(lon=0, lat=-40, fill_value=-0.5) + (
        features[2] + features[4] + features[5] + features[6]
    )
    da_shift_03 = da.shift(lon=0, lat=-40, fill_value=-0.5) + (
        features[2] + features[5] + features[6]
    )

    Anom = xr.concat(
        (
            da,
            da_shift_01,
            da_shift_02,
            da_shift_03,
        ),
        dim="time",
    )

    Anom["time"] = np.arange(1, 5)
    Anom = Anom.where(Anom > 0, drop=False, other=0)

    mask = xr.DataArray(np.ones(Anom[0, :, :].shape), coords=Anom[0, :, :].coords)
    mask[60:90, 120:190] = 0

    return Anom, mask


@pytest.mark.parametrize("radius", [8, 10])
@pytest.mark.parametrize("min_size_quartile", [0.75, 0.80])
@pytest.mark.parametrize("timedim", ["time", "t", "months"])
@pytest.mark.parametrize("xdim", ["lon", "longitude", "whoo"])
@pytest.mark.parametrize("ydim", ["lat", "latitude", "whaa"])
@pytest.mark.parametrize("positive", [True, False])
def test_track(example_data, radius, min_size_quartile, timedim, xdim, ydim, positive):
    Anom, mask = example_data
    if positive == False:
        Anom = Anom * -1
    Anom = Anom.rename({"time": timedim, "lon": xdim, "lat": ydim})
    mask = mask.rename({"time": timedim, "lon": xdim, "lat": ydim})

    tracker = Tracker(
        Anom,
        mask,
        radius,
        min_size_quartile,
        timedim=timedim,
        xdim=xdim,
        ydim=ydim,
        positive=positive,
    )
    new_labels = tracker.track()
    assert (
        new_labels.attrs["percent area reject"]
        + new_labels.attrs["percent area accept"]
    ) == 1.0

    if positive == True:
        assert Anom.sum() >= 0

    if positive == False:
        assert Anom.sum() <= 0


def test_morphological_operations(
    example_data,
    radius=8,
    min_size_quartile=0.75,
    timedim="time",
    xdim="lat",
    ydim="lon",
    positive=True,
):
    Anom, mask = example_data
    tracker = Tracker(
        Anom.chunk({"time": 1}),
        mask,
        radius,
        min_size_quartile,
        timedim,
        xdim,
        ydim,
        positive,
    )
    binary_images = tracker._morphological_operations()

    assert (timedim, ydim, xdim) != Anom.expand_dims("level").dims
    assert isinstance(binary_images.data, dsa.Array)

    ocetrac_guess = Anom.where(binary_images == True, drop=False, other=np.nan)
    best_guess = Anom.where(Anom > 0, drop=False, other=np.nan)
    part = ocetrac_guess.isin(best_guess)
    whole = best_guess.isin(best_guess)

    print(part.sum().values / whole.sum().values * 10)

    assert part.sum().values / whole.sum().values * 100 >= 80
    assert part.sum().values == 26122


def test_apply_mask(
    example_data,
    radius=8,
    min_size_quartile=0.75,
    timedim="time",
    xdim="lat",
    ydim="lon",
    positive=True,
):
    Anom, mask = example_data
    tracker = Tracker(
        Anom.chunk({"time": 1}),
        mask,
        radius,
        min_size_quartile,
        timedim,
        xdim,
        ydim,
        positive,
    )
    binary_images = tracker._morphological_operations()
    mask = mask.broadcast_like(binary_images)
    binary_images_with_mask = _apply_mask(binary_images, mask)
    assert (binary_images_with_mask.where(mask == 0, drop=True) == 0).all()


def test_filter_area(
    example_data,
    radius=8,
    min_size_quartile=0.75,
    timedim="time",
    xdim="lat",
    ydim="lon",
    positive=True,
):
    Anom, mask = example_data
    tracker = Tracker(
        Anom.chunk({"time": 1}),
        mask,
        radius,
        min_size_quartile,
        timedim,
        xdim,
        ydim,
        positive,
    )
    binary_images = tracker._morphological_operations()
    binary_images_with_mask = _apply_mask(binary_images, mask)
    area, min_area, binary_labels, N_initial = tracker._filter_area(
        binary_images_with_mask
    )

    assert min_area == 2761.5
    assert N_initial.astype(int) == 15


def test_label_either(
    example_data,
    radius=8,
    min_size_quartile=0.75,
    timedim="time",
    xdim="lat",
    ydim="lon",
    positive=True,
):
    Anom, mask = example_data
    tracker = Tracker(
        Anom.chunk({"time": 1}),
        mask,
        radius,
        min_size_quartile,
        timedim,
        xdim,
        ydim,
        positive,
    )
    binary_images = tracker._morphological_operations()
    binary_images_with_mask = _apply_mask(binary_images, mask)
    labels, num = tracker._label_either(
        binary_images_with_mask, return_num=True, connectivity=3
    )

    assert labels[2, :, :].max() == 6.0
    assert all([i in labels[2, :, :] for i in range(0, 6)])


def test_wrap(
    example_data,
    radius=8,
    min_size_quartile=0.75,
    timedim="time",
    xdim="lat",
    ydim="lon",
    positive=True,
):
    Anom, mask = example_data
    tracker = Tracker(
        Anom.chunk({"time": 1}),
        mask,
        radius,
        min_size_quartile,
        timedim,
        xdim,
        ydim,
        positive,
    )
    binary_images = tracker._morphological_operations()
    binary_images_with_mask = _apply_mask(binary_images, mask)
    area, min_area, binary_labels, N_initial = tracker._filter_area(
        binary_images_with_mask
    )
    labels, num = tracker._label_either(
        binary_images_with_mask, return_num=True, connectivity=3
    )
    labels_wrapped, N_final = tracker._wrap(labels)

    assert N_final == 6


def test_nan_timestep(
    example_data,
    radius=8,
    min_size_quartile=0.75,
    timedim="time",
    xdim="lat",
    ydim="lon",
    positive=True,
):
    Anom, mask = example_data
    Anom.loc[{"time": 2}] = Anom.sel(time=2) * 0
    tracker = Tracker(
        Anom.chunk({"time": 1}),
        mask,
        radius,
        min_size_quartile,
        timedim,
        xdim,
        ydim,
        positive,
    )
    binary_images = tracker._morphological_operations()
    binary_images_with_mask = _apply_mask(binary_images, mask)
    area, min_area, binary_labels, N_initial = tracker._filter_area(
        binary_images_with_mask
    )
    # PR 33 fixes issue where a timestep with no objects resulted in
    # no labeled objects at all future timesteps.
    labels_in_time = binary_labels.max(("lat", "lon"))
    assert labels_in_time.sel(time=2) == 0
    assert labels_in_time.sel(time=3) != 0
